﻿using UnityEngine.TestTools;
using NUnit.Framework;
using System.Collections;
using Windows.Kinect;
using System.Runtime.InteropServices;
using KinectModule;
using System.Collections.Generic;
using System.Linq;
using System;

// To test the update function and the updated buffer, you need a stub for the source frame class.

public class SourceBufferTests
{
    [Test]
    public void ColorBuffer_NotEqual_Null()
    {
        var obj = new SourceBuffer();

        var expected = null as byte[];
        var actual = obj.ColorBuffer;

        Assert.AreNotEqual(expected, actual, "ColorBuffer should not be null value.");
    }

    [Test]
    public void BodyIndexBuffer_NotEqual_Null()
    {
        var obj = new SourceBuffer();

        var expected = null as byte[];
        var actual = obj.BodyIndexBuffer;

        Assert.AreNotEqual(expected, actual, "BodyIndexBuffer should not be null value.");
    }

    [Test]
    public void DepthBuffer_NotEqual_Null()
    {
        var obj = new SourceBuffer();

        var expected = null as ushort[];
        var actual = obj.DepthBuffer;

        Assert.AreNotEqual(expected, actual, "DepthBuffer should not be null value.");
    }

    [Test]
    public void BodyBuffer_NotEqual_Null()
    {
        var obj = new SourceBuffer();

        var expected = null as Body[];
        var actual = obj.BodyBuffer;

        Assert.AreNotEqual(expected, actual, "BodyBuffer should not be null value.");
    }

    [Test]
    public void ColorBuffer_Equal_Green()
    {
        var frame = new GreenFrameStub();

        var obj = new SourceBuffer();
        obj.UpdateBuffers(frame);

        var expected = new byte[KinectConstants.ColorWidth *
                                KinectConstants.ColorHeight *
                                KinectConstants.ColorCount];
        frame.LastColorFrame.CopyData(expected);

        var actual = obj.ColorBuffer;

        Assert.AreEqual(expected, actual, "Colorbuffer should be filled with green.");
    }
}

public class GreenFrameStub : IMultiSourceFrame
{
    private GreenColorFrameStub _ColorFrame = new GreenColorFrameStub();
    private BodyIndexFrameDummy _BodyIndexFrame = new BodyIndexFrameDummy();
    private DepthFrameDummy _DepthFrame = new DepthFrameDummy();
    private BodyFrameDummy _BodyFrame = new BodyFrameDummy();

    public IColorFrame LastColorFrame
    { get { return _ColorFrame; } }
    public IBodyIndexFrame LastBodyIndexFrame
    { get { return _BodyIndexFrame; } }
    public IDepthFrame LastDepthFrame
    { get { return _DepthFrame; } }
    public IBodyFrame LastBodyFrame
    { get { return _BodyFrame; } }

    private class GreenColorFrameStub : IColorFrame
    {
        private static uint[] data =
            Enumerable.Repeat<uint>(0x00FF00FF,
                                    KinectConstants.ColorWidth *
                                    KinectConstants.ColorHeight).ToArray<uint>();

        public void CopyData(byte[] buffer)
        {
            Buffer.BlockCopy(buffer, 0, data, 0, buffer.Length);
        }
    }

    private class BodyIndexFrameDummy : IBodyIndexFrame
    {
        public void CopyData(byte[] buffer)
        { }
    }

    private class DepthFrameDummy : IDepthFrame
    {
        public void CopyData(ushort[] buffer)
        { }
    }

    private class BodyFrameDummy : IBodyFrame
    {
        public void CopyData(Body[] buffer)
        { }
    }
}